@@ -20,10 +20,17 @@ import stevedore
 
 from networking_generic_switch import exceptions as gsw_exc
 
-
 GENERIC_SWITCH_NAMESPACE = 'generic_switch.devices'
 LOG = logging.getLogger(__name__)
 
+# Internal ngs options will not be passed to driver.
+NGS_INTERNAL_OPTS = [
+    {'name': 'ngs_mac_address'},
+    {'name': 'ngs_ssh_connect_timeout', 'default': 60},
+    {'name': 'ngs_ssh_connect_interval', 'default': 10},
+    {'name': 'ngs_max_connections', 'default': 1},
+]
+
 
 def device_manager(device_cfg):
     device_type = device_cfg.get('device_type', '')
@@ -55,6 +62,15 @@ def _load_failure_hook(manager, entrypoint, exception):
 class GenericSwitchDevice(object):
 
     def __init__(self, device_cfg):
+        self.ngs_config = {}
+        self.config = {}
+        # Do not expose NGS internal options to device config.
+        for opt in NGS_INTERNAL_OPTS:
+            opt_name = opt['name']
+            if opt_name in device_cfg.keys():
+                self.ngs_config[opt_name] = device_cfg.pop(opt_name)
+            elif 'default' in opt:
+                self.ngs_config[opt_name] = opt['default']
         self.config = device_cfg
 
     @abc.abstractmethod
