@@ -24,6 +24,9 @@ class TestNetmikoAristaEos(test_netmiko_base.NetmikoSwitchTestBase):
         device_cfg = {'device_type': 'netmiko_arista_eos'}
         return arista.AristaEos(device_cfg)
 
+    def test_constants(self):
+        self.assertEqual(('write memory',), self.switch.SAVE_CONFIGURATION)
+
     @mock.patch('networking_generic_switch.devices.netmiko_devices.'
                 'NetmikoSwitch.send_commands_to_device')
     def test_add_network(self, m_exec):
